@@ -21,8 +21,6 @@ from pyfstat.core import (
     args,
     DefunctClass,
 )
-import lalpulsar
-import lal
 
 
 class GridSearch(BaseSearchClass):
@@ -65,6 +63,9 @@ class GridSearch(BaseSearchClass):
     }
     """Formatted labels used for annotating central values in plots."""
 
+    fmt_detstat = "%.9g"
+    """Standard output precision for detection statistics."""
+
     @helper_functions.initializer
     def __init__(
         self,
@@ -118,16 +119,24 @@ class GridSearch(BaseSearchClass):
             os.mkdir(outdir)
         self.set_out_file()
         self.search_keys = ["F0", "F1", "F2", "Alpha", "Delta"]
-        self.output_keys = self.search_keys.copy()
+        for k in self.search_keys:
+            setattr(self, k, np.atleast_1d(getattr(self, k + "s")))
         if self.BSGL:
             self.detstat = "log10BSGL"
         else:
             self.detstat = "twoF"
-        self.output_keys.append(self.detstat)
-        for k in self.search_keys:
-            setattr(self, k, np.atleast_1d(getattr(self, k + "s")))
+        self._initiate_search_object()
+        self._set_output_keys()
         self.output_file_header = self.get_output_file_header()
 
+    def _set_output_keys(self):
+        self.output_keys = self.search_keys.copy()
+        self.output_keys.append("twoF")
+        if hasattr(self.search, "twoFX"):
+            self.output_keys += [f"twoF{IFO}" for IFO in self.search.detector_names]
+        if self.BSGL:
+            self.output_keys.append(self.detstat)
+
     def _get_search_ranges(self):
         if (self.minCoverFreq is None) or (self.maxCoverFreq is None):
             return {key: getattr(self, key + "s") for key in self.search_keys}
@@ -155,7 +164,6 @@ class GridSearch(BaseSearchClass):
                 earth_ephem=self.earth_ephem,
                 sun_ephem=self.sun_ephem,
             )
-            self.search.get_det_stat = self.search.get_fullycoherent_twoF
         else:
             self.search = SemiCoherentSearch(
                 label=self.label,
@@ -172,7 +180,6 @@ class GridSearch(BaseSearchClass):
                 detectors=self.detectors,
                 injectSources=self.injectSources,
             )
-            self.search.get_det_stat = self.search.get_semicoherent_det_stat
         # make sure to overwrite the min/max starttime in case the user
         # passed None and they were read from SFTs
         self.minStartTime = self.search.minStartTime
@@ -365,9 +372,6 @@ class GridSearch(BaseSearchClass):
                 self.data = old_data
                 return
 
-        if hasattr(self, "search") is False:
-            self._initiate_search_object()
-
         logging.info(
             "Running search over a total of {:d} grid points...".format(
                 np.shape(iterable)[0]
@@ -383,8 +387,13 @@ class GridSearch(BaseSearchClass):
         for n, vals in enumerate(
             tqdm(iterable, total=getattr(self, "total_iterations", None))
         ):
+            thisCand = list(vals)
             detstat = self.search.get_det_stat(*vals)
-            thisCand = list(vals) + [detstat]
+            thisCand.append(self.search.twoF)
+            if hasattr(self.search, "twoFX"):
+                thisCand += list(self.search.twoFX[: self.search.numDetectors])
+            if self.detstat != "twoF":
+                thisCand.append(detstat)
             for k, key in enumerate(self.output_keys):
                 data[key][n] = thisCand[k]
 
@@ -397,7 +406,12 @@ class GridSearch(BaseSearchClass):
     def _get_savetxt_fmt_dict(self):
         """Define the output precision for each parameter and computed quantity."""
         fmt_dict = helper_functions.get_doppler_params_output_format(self.output_keys)
-        fmt_dict[self.detstat] = "%.9g"
+        fmt_dict["twoF"] = self.fmt_detstat
+        if hasattr(self.search, "twoFX"):
+            for IFO in self.search.detector_names:
+                fmt_dict[f"twoF{IFO}"] = self.fmt_detstat
+        if self.BSGL:
+            fmt_dict["log10BSGL"] = self.fmt_detstat
         return fmt_dict
 
     def _get_savetxt_fmt_list(self):
@@ -722,6 +736,20 @@ class GridSearch(BaseSearchClass):
         else:
             return ax
 
+    def get_max_det_stat(self):
+        """Get the maximum detection statistic over the grid.
+
+        This requires the `run()` method to have been called before.
+
+        Returns
+        -------
+        d: dict
+            Dictionary containing parameters and detection statistic at the maximum.
+        """
+        idx = np.argmax(self.data[self.detstat])
+        d = OrderedDict([(key, self.data[key][idx]) for key in self.output_keys])
+        return d
+
     def get_max_twoF(self):
         """Get the maximum twoF over the grid.
 
@@ -891,18 +919,14 @@ class TransientGridSearch(GridSearch):
             os.mkdir(outdir)
         self.set_out_file()
         self.search_keys = ["F0", "F1", "F2", "Alpha", "Delta"]
-        self.output_keys = self.search_keys.copy()
+        for k in self.search_keys:
+            setattr(self, k, np.atleast_1d(getattr(self, k + "s")))
         if self.BSGL:
             self.detstat = "log10BSGL"
         else:
-            self.detstat = "twoF"
-        self.output_keys.append(self.detstat)
-        # for consistency below, t0/tau must come after detstat
-        # they are not included in self.search_keys because the main Fstat
-        # code does not loop over them
-        self.output_keys += ["t0", "tau"]
-        for k in self.search_keys:
-            setattr(self, k, np.atleast_1d(getattr(self, k + "s")))
+            self.detstat = "maxTwoF"
+        self._initiate_search_object()
+        self._set_output_keys()
         self.output_file_header = self.get_output_file_header()
         if self.outputTransientFstatMap:
             self.tCWfilebase = os.path.splitext(self.out_file)[0] + "_tCW_"
@@ -911,6 +935,23 @@ class TransientGridSearch(GridSearch):
                 " results to {}*.dat".format(self.tCWfilebase)
             )
 
+    def _set_output_keys(self):
+        self.output_keys = self.search_keys.copy()
+        self.output_keys.append("twoF")
+        if hasattr(self.search, "twoFX"):
+            self.output_keys += [f"twoF{IFO}" for IFO in self.search.detector_names]
+        self.output_keys.append("maxTwoF")
+        if hasattr(self.search, "twoFXatMaxTwoF"):
+            self.output_keys += [
+                f"twoF{IFO}atMaxTwoF" for IFO in self.search.detector_names
+            ]
+        if self.detstat != "maxTwoF":
+            self.output_keys.append(self.detstat)
+        # for consistency below, t0/tau must come after detstat
+        # they are not included in self.search_keys because the main Fstat
+        # code does not loop over them
+        self.output_keys += ["t0", "tau"]
+
     def _initiate_search_object(self):
         logging.info("Setting up search object")
         search_ranges = self._get_search_ranges()
@@ -940,7 +981,6 @@ class TransientGridSearch(GridSearch):
             earth_ephem=self.earth_ephem,
             sun_ephem=self.sun_ephem,
         )
-        self.search.get_det_stat = self.search.get_fullycoherent_twoF
         # make sure to overwrite the min/max starttime in case the user
         # passed None and they were read from SFTs
         self.minStartTime = self.search.minStartTime
@@ -975,9 +1015,6 @@ class TransientGridSearch(GridSearch):
             self.data = old_data
             return
 
-        if hasattr(self, "search") is False:
-            self._initiate_search_object()
-
         output_dtype = np.dtype(
             {
                 "names": self.output_keys,
@@ -992,32 +1029,29 @@ class TransientGridSearch(GridSearch):
             )
         )
         for n, vals in enumerate(tqdm(self.input_data)):
+            thisCand = list(vals)
             detstat = self.search.get_det_stat(*vals)
             windowRange = getattr(self.search, "windowRange", None)
-            FstatMap = getattr(self.search, "FstatMap", None)
             self.timingFstatMap += getattr(self.search, "timingFstatMap", 0.0)
-            thisCand = list(vals) + [detstat]
+            thisCand.append(self.search.twoF)
+            if hasattr(self.search, "twoFX"):
+                thisCand += list(self.search.twoFX[: self.search.numDetectors])
+            thisCand.append(self.search.maxTwoF)
+            if hasattr(self.search, "twoFXatMaxTwoF"):
+                thisCand += list(self.search.twoFXatMaxTwoF[: self.search.numDetectors])
+            if self.detstat != "maxTwoF":
+                thisCand.append(detstat)
             if getattr(self, "transientWindowType", None):
-                if self.tCWFstatMapVersion == "lal":
-                    F_mn = FstatMap.F_mn.data
-                else:
-                    F_mn = FstatMap.F_mn
+                if not hasattr(self.search, "FstatMap"):
+                    raise RuntimeError(
+                        "Since transientWindowType!=None, we expected to have a FstatMap."
+                    )
                 if self.outputTransientFstatMap:
                     tCWfile = self.get_transient_fstat_map_filename(thisCand)
-                    if self.tCWFstatMapVersion == "lal":
-                        fo = lal.FileOpen(tCWfile, "w")
-                        for hline in self.output_file_header:
-                            lal.FilePuts("# {:s}\n".format(hline), fo)
-                        lal.FilePuts("# t0[s]      tau[s]      2F\n", fo)
-                        lalpulsar.write_transientFstatMap_to_fp(
-                            fo, FstatMap, windowRange, None
-                        )
-                        # instead of lal.FileClose(),
-                        # which is not SWIG-exported:
-                        del fo
-                    else:
-                        self.write_F_mn(tCWfile, F_mn, windowRange)
-                maxidx = np.unravel_index(F_mn.argmax(), F_mn.shape)
+                    self.search.FstatMap.write_F_mn_to_file(
+                        tCWfile, windowRange, self.output_file_header
+                    )
+                maxidx = self.search.FstatMap.get_maxF_idx()
                 thisCand += [
                     windowRange.t0 + maxidx[0] * windowRange.dt0,
                     windowRange.tau + maxidx[1] * windowRange.dtau,
@@ -1070,35 +1104,20 @@ class TransientGridSearch(GridSearch):
     def _get_savetxt_fmt_dict(self):
         """Define the output precision for each parameter and computed quantity."""
         fmt_dict = helper_functions.get_doppler_params_output_format(self.output_keys)
-        fmt_dict[self.detstat] = "%.9g"
+        fmt_dict["twoF"] = self.fmt_detstat
+        if hasattr(self.search, "twoFX"):
+            for IFO in self.search.detector_names:
+                fmt_dict[f"twoF{IFO}"] = self.fmt_detstat
+        fmt_dict["maxTwoF"] = self.fmt_detstat
+        if hasattr(self.search, "twoFXatMaxTwoF"):
+            for IFO in self.search.detector_names:
+                fmt_dict[f"twoF{IFO}atMaxTwoF"] = self.fmt_detstat
+        if self.detstat != "maxTwoF":
+            fmt_dict[self.detstat] = self.fmt_detstat
         fmt_dict["t0"] = "%d"
         fmt_dict["tau"] = "%d"
         return fmt_dict
 
-    def write_F_mn(self, tCWfile, F_mn, windowRange):
-        """Helper function to format a transient-F-stat matrix over `(t0,tau)`.
-
-        Parameters
-        ----------
-        tCWfile: str
-            Name of the file to write to.
-        F_mn: np.ndarray
-            The 2D matrix of transient twoF-stat values.
-        windowRange: lalpulsar.transientWindowRange_t
-            A lalpulsar structure containing the transient parameters.
-        """
-        with open(tCWfile, "w") as tfp:
-            for hline in self.output_file_header:
-                tfp.write("# {:s}\n".format(hline))
-            tfp.write("# t0 [s]     tau [s]     2F\n")
-            for m, F_m in enumerate(F_mn):
-                this_t0 = windowRange.t0 + m * windowRange.dt0
-                for n, this_F in enumerate(F_m):
-                    this_tau = windowRange.tau + n * windowRange.dtau
-                    tfp.write(
-                        "  %10d %10d %- 11.8g\n" % (this_t0, this_tau, 2.0 * this_F)
-                    )
-
     def __del__(self):
         if hasattr(self, "search"):
             self.search.__del__()
@@ -1166,11 +1185,13 @@ class GridGlitchSearch(GridSearch):
         """
 
         self._set_init_params_dict(locals())
+        if os.path.isdir(outdir) is False:
+            os.mkdir(outdir)
+        self.set_out_file()
         self.BSGL = False
         self.input_arrays = False
         if tglitchs is None:
             raise ValueError("You must specify `tglitchs`")
-
         self.search_keys = [
             "F0",
             "F1",
@@ -1181,33 +1202,30 @@ class GridGlitchSearch(GridSearch):
             "delta_F1",
             "tglitch",
         ]
-        self.output_keys = self.search_keys.copy()
-        self.detstat = "twoF"
-        self.output_keys += [self.detstat]
         for k in self.search_keys:
             setattr(self, k, np.atleast_1d(getattr(self, k + "s")))
-        search_ranges = self._get_search_ranges()
+        self.detstat = "twoF"
+        self._initiate_search_object()
+        self._set_output_keys()
+        self.output_file_header = self.get_output_file_header()
 
+    def _initiate_search_object(self):
+        logging.info("Setting up search object")
+        search_ranges = self._get_search_ranges()
         self.search = SemiCoherentGlitchSearch(
-            label=label,
-            outdir=outdir,
+            label=self.label,
+            outdir=self.outdir,
             sftfilepattern=self.sftfilepattern,
-            tref=tref,
-            minStartTime=minStartTime,
-            maxStartTime=maxStartTime,
-            minCoverFreq=minCoverFreq,
-            maxCoverFreq=maxCoverFreq,
+            tref=self.tref,
+            minStartTime=self.minStartTime,
+            maxStartTime=self.maxStartTime,
+            minCoverFreq=self.minCoverFreq,
+            maxCoverFreq=self.maxCoverFreq,
             search_ranges=search_ranges,
             BSGL=self.BSGL,
-            earth_ephem=earth_ephem,
-            sun_ephem=sun_ephem,
+            earth_ephem=self.earth_ephem,
+            sun_ephem=self.sun_ephem,
         )
-        self.search.get_det_stat = self.search.get_semicoherent_nglitch_twoF
-
-        if os.path.isdir(outdir) is False:
-            os.mkdir(outdir)
-        self.set_out_file()
-        self.output_file_header = self.get_output_file_header()
 
     def _get_savetxt_fmt_dict(self):
         """Define the output precision for each parameter and computed quantity."""
@@ -1215,7 +1233,7 @@ class GridGlitchSearch(GridSearch):
         fmt_dict["delta_F0"] = "%.16g"
         fmt_dict["delta_F1"] = "%.16g"
         fmt_dict["tglitch"] = "%d"
-        fmt_dict[self.detstat] = "%.9g"
+        fmt_dict[self.detstat] = self.fmt_detstat
         return fmt_dict
 
 
